#include <pic.h>
#include <htc.h>
#include <stdlib.h>

__CONFIG(XT&WDTDIS&LVPDIS);

void lcd_line1(void);			//function prototypes
void lcd_line2(void);
void lcd_cmd(unsigned char);
void lcd_char(char);
void e_togg(void);
void lcd_init(void);
void lcd_string(char *);
void lcd_busy(void);
unsigned char read_a2d(void);
void lcd_16number(int);
void delay(void);
void clk_write(unsigned char);
unsigned char clk_read(void);
void clkburst(void);
void datafix(void);

#define	E		RD2
#define	RS		RD0
#define RW		RD1
#define	busyflag	RB3
#define RW_TrisBit	TRISD1
#define D7_TrisBit	TRISB3

#define	RST		RD4
#define	DATA		RD5
#define	SCLK		RD6
#define DATA_Tris	TRISD5

char number[] = "  :  :          ";
char daydate[] = "  :  :20        ";
char sec[] = "           ";

void main(void)
{
	char x;
	unsigned char a,b;
	a=0;
	number[2] = number[5] = 0x3a;
	daydate[2] = daydate[5] = 0x3a;
	daydate[6] = 0x32;
	daydate[7] = 0x30;
	TRISB = TRISD = 0;
	RST = 0;				//1302 reset low
	SCLK = 0;				//1302 clock low
	RW = 0;					//set lcd R/W low
	E = 0;					//set lcd E low
	lcd_busy();				//wait for LCD to settle
	delay();
	lcd_init();

	while(1)
	{
		RST = 1;			//read hours
		clk_write(0xbf);		//burst read
		sec[0] = clk_read();		//sec
		sec[1] = clk_read();		//min		
		sec[2] = clk_read();		//hour
		sec[3] = clk_read();		//date
		sec[4] = clk_read();		//month
		sec[5] = clk_read();		//day
		sec[6] = clk_read();		//year
		sec[7] = clk_read();		//control
		RST = 0;
		datafix();			//convert for display
		lcd_line1();			//and print to LCD
		lcd_string(number);
		lcd_line2();
		lcd_string(daydate);
		delay();
	}
}

void datafix(void)
{
	char temp,x;
	char *foo = sec;
	char bleh[] = "                ";
	for(x=0;x<14;x+=2)
	{
		temp = *foo;
		temp &+ 0xf0;
		temp >>= 4;
		bleh[x] = temp + 0x30;
		temp = *foo;
		temp &= 0x0f;
		bleh[x+1] = temp + 0x30;
		foo++;
	}
	number[6]=bleh[0];		//sec
	number[7]=bleh[1];
	number[3]=bleh[2];		//min
	number[4]=bleh[3];
	number[0]=bleh[4];		//hour
	number[1]=bleh[5];
	daydate[3]=bleh[6];		//day
	daydate[4]=bleh[7];
	daydate[0]=bleh[8];		//month
	daydate[1]=bleh[9];
	daydate[8]=bleh[12];		//year
	daydate[9]=bleh[13];
}

void clk_write(unsigned char dat)
{
	char x;
	for(x=0;x<8;x++)
	{
		if(dat & 0x01)				//put bit on data pin
		{
			DATA = 1;
		}
		else DATA = 0;
		SCLK = 1;				//toggle clock to latch bit
		SCLK = 0;
		dat >>= 1;				//move next bit into place
	}
}

unsigned char clk_read(void)
{
	char x;
	unsigned char dat,tmpdat;
	dat = 0;
	DATA_Tris = 1;					//make DATA input
	for(x=0;x<8;x++)
	{
		if(DATA)				//read a bit
		{
			tmpdat |= 1;
		}
		else tmpdat |= 0;
		tmpdat <<= 7;
		dat >>= 1;
		dat |= tmpdat;
		SCLK = 1;				//toggle clock to latch bit
		SCLK = 0;
	}
	DATA_Tris = 0;
	return(dat);
}

void lcd_string(char *senpoint)
{
	while(*senpoint != '\0')
	{
		lcd_char(*senpoint);
		senpoint++;
	}
}	

void lcd_busy(void)
{
	RW_TrisBit = 1;					//make R/W input (read)
	D7_TrisBit = 1;					//make D7 input
	RS = 0;						//set RS low
	RW = 1;						//set R/W high
	E = 1;						//set E high
	while(busyflag);				//wait for busy flag to go low
	E = 0;						//set E low
	RW = 0;						//set R/W low
	TRISB = 0;					//make D7 output
	RW_TrisBit = 0;					//make R/W output (write)
}	
	
void lcd_16number(int num)
{
	number[0]=(char)(abs(num/10000)+0x30);
	num=num-abs(num/10000)*10000;
	number[1]=(char)(abs(num/1000)+0x30);
	num=num-abs(num/1000)*1000;
	number[2]=(char)(abs(num/100)+0x30);
	num=num-abs(num/100)*100;
	number[3]=(char)(abs(num/10)+0x30);
	num=num-abs(num/10)*10;
	number[4]=(char)(num+0x30);
}

void lcd_line1(void)
{
	lcd_cmd(0x80);
}

void lcd_line2(void)
{
	lcd_cmd(0xc0);
}		

void lcd_cmd(unsigned char letter)
{
	lcd_busy();
	PORTB = letter;					//put char in PORTB
	PORTB = PORTB >> 4;				//shift over to output high 4 bits
	RS = 0;						//RS low
	e_togg();					//latch the data
	lcd_busy();
	PORTB = letter;
	RS = 0;						//RS low
	e_togg();					//latch it
}

void lcd_char(char letter)
{
	lcd_busy();
	PORTB = letter;					//put char in PORTB
	PORTB = PORTB >> 4;				//shift over to output high 4 bits
	RS = 1;						//RS high
	e_togg();					//latch the data
	lcd_busy();
	PORTB = letter;
	RS = 1;						//RS high
	e_togg();					//latch it
}

void lcd_init(void)
{
	PORTB = 0x03;					//send 3
	e_togg();
	lcd_busy();
	PORTB = 0x03;
	e_togg();
	lcd_busy();
	PORTB = 0x03;
	e_togg();
	lcd_busy();
	PORTB = 0x02;					//send 2 - set 4-bit mode
	e_togg();
	lcd_busy();
	lcd_cmd(0x28);					//set 4-bit mode and 2 lines
	lcd_busy();
	lcd_cmd(0x10);					//cursor move & shift left
	lcd_busy();
	lcd_cmd(0x06);					//entry mode = increment
	lcd_busy();
	lcd_cmd(0x0c);					//display on - cursor blink on
	lcd_busy();
	lcd_cmd(0x01);					//clear display
	lcd_busy();
}

void e_togg(void)
{
	E=1;
	E=0;
}

void delay(void)
{
	int x,y;
	for(x=0;x<1;x++)
	{
		for(y=0;y<10000;y++){}
	}
}

